//
//  TADCustomEvent.swift
//  AdmobSDKTest
//
//  Created by Charles Wang on 2023/2/16.
//

import Foundation
import GoogleMobileAds
import TAMediaAdsFramework

class TADCustomEvent: NSObject, GADMediationAdapter {
    private var customEventNative: TADCustomEventNative?
    private var customEventBanner: TADCustomEventBanner?
    private var customEventInterstitial: TADCustomEventInterstitial?
    
    override required init() {
        super.init()
    }
    
    deinit {
        print("TADCustomEvent deinit")
    }
    
    static func adapterVersion() -> GADVersionNumber {
        return GADVersionNumber(majorVersion: 1, minorVersion: 1, patchVersion: 0)
    }
    
    static func adSDKVersion() -> GADVersionNumber {
        let version = TADMobileAds.getSDKVersion().components(separatedBy: ".")
        let major = Int(version[0]) ?? 0
        let minor = Int(version[1]) ?? 0
        let patch = Int(version[2]) ?? 0
        return GADVersionNumber(majorVersion: major, minorVersion: minor, patchVersion: patch)
    }
    
    static func networkExtrasClass() -> GADAdNetworkExtras.Type? {
        return nil
    }
    
    static func setUpWith(_ configuration: GADMediationServerConfiguration, completionHandler: @escaping GADMediationAdapterSetUpCompletionBlock) {
        TADMobileAds.config()
        completionHandler(nil)
    }
    
    func loadBanner(for adConfiguration: GADMediationBannerAdConfiguration, completionHandler: @escaping GADMediationBannerLoadCompletionHandler) {
        customEventBanner = TADCustomEventBanner()
        customEventBanner?.loadBanner(for: adConfiguration, completionHandler: completionHandler)
    }
    
    func loadInterstitial(for adConfiguration: GADMediationInterstitialAdConfiguration, completionHandler: @escaping GADMediationInterstitialLoadCompletionHandler) {
        customEventInterstitial = TADCustomEventInterstitial()
        customEventInterstitial?.loadInterstitial(for: adConfiguration, completionHandler: completionHandler)
    }
    
    func loadNativeAd(for adConfiguration: GADMediationNativeAdConfiguration, completionHandler: @escaping GADMediationNativeLoadCompletionHandler) {
        customEventNative = TADCustomEventNative()
        customEventNative?.loadNativeAd(for: adConfiguration, completionHandler: completionHandler)
    }
}
