//
//  TADCustomEventBanner.swift
//  AdmobSDKTest
//
//  Created by Charles Wang on 2023/2/16.
//

import Foundation
import GoogleMobileAds
import TAMediaAdsFramework

class TADCustomEventBanner: NSObject, GADMediationBannerAd {
    deinit {
        print("CustomBanner - deinit!!")
        bannerView?.delegate = nil
        bannerView = nil
    }
    
    var view: UIView {
      return bannerView ?? UIView()
    }
    
    var bannerView: TADBannerView?

    /// The ad event delegate to forward ad rendering events to the Google Mobile Ads SDK.
    var delegate: GADMediationBannerAdEventDelegate?

    /// Completion handler called after ad load
    var completionHandler: GADMediationBannerLoadCompletionHandler?
    
    func loadBanner(for adConfiguration: GADMediationBannerAdConfiguration,
        completionHandler: @escaping GADMediationBannerLoadCompletionHandler) {
        
        let tadRequest = TADRequest()
        tadRequest.isTesting = adConfiguration.isTestRequest
        tadRequest.setLocationWithLatitude(adConfiguration.userLatitude, longitude: adConfiguration.userLongitude, accuracy: adConfiguration.userLocationAccuracyInMeters)
            let adSize = adConfiguration.adSize
        let adRatio: CGFloat = floor((adSize.size.width/adSize.size.height)*100)/100
        let ta1200x627AdRatio: CGFloat = floor((1200.0/627)*100)/100

        if GADAdSizeEqualToSize(adSize, GADAdSizeBanner) {
            self.bannerView = TADBannerView(adSize: kTADAdSizeBanner)
        } else if GADAdSizeEqualToSize(adSize, GADAdSizeMediumRectangle) {
            self.bannerView = TADBannerView(adSize: kTADAdSizeMediumRectangle)
        } else if adRatio == ta1200x627AdRatio {
            self.bannerView = TADBannerView(adSize: TADAdSize1200x627RatioBannerWithWidth(adSize.size.width))
        } else {
            let error = NSError(domain: "com.TAmedia.TADCustomBanner", code: GADErrorCode.mediationInvalidAdSize.rawValue, userInfo: nil)
            delegate = completionHandler(nil, error)
        }
        
        self.completionHandler = completionHandler
        self.bannerView?.delegate = self
        self.bannerView?.adUnitID = adConfiguration.credentials.settings["parameter"] as? String
        self.bannerView?.turnOnMicrophone = false
        self.bannerView?.rootViewController = adConfiguration.topViewController
        self.bannerView?.load(tadRequest)
    }
}


// MARK: TADBannerViewDelegate

extension TADCustomEventBanner: TADBannerViewDelegate {
    func adViewDidReceiveAd(_ view: TADBannerView!) {
        print("CustomBanner - adViewDidReceiveAd!!")
        if let handler = completionHandler {
            delegate = handler(self, nil)
        }
    }

    func adView(_ view: TADBannerView!, didFailToReceiveAdWithError error: TADRequestError!) {
        print("CustomBanner - didFailToReceiveAdWithError!! error: \(String(describing: error))")
        if let handler = completionHandler {
            delegate = handler(nil, error)
        }
    }

    func adViewWillPresentScreen(_ adView: TADBannerView!) {
        print("CustomBanner - adViewWillPresentScreen!!")
        delegate?.reportImpression()
    }

    func adViewWillDismissScreen(_ adView: TADBannerView!) {
        print("CustomBanner - adViewWillDismissScreen!!")
    }

    func adViewDidDismissScreen(_ adView: TADBannerView!) {
        print("CustomBanner - adViewDidDismissScreen!!")
    }

    func adViewWillLeaveApplication(_ adView: TADBannerView!) {
        print("CustomBanner - adViewWillLeaveApplication!!")
        delegate?.reportClick()
    }
}
