//
//  TADCustomEventInterstitial.swift
//  AdmobSDKTest
//
//  Created by Charles Wang on 2023/2/16.
//

import Foundation
import GoogleMobileAds
import TAMediaAdsFramework

class TADCustomEventInterstitial: NSObject, GADMediationInterstitialAd {
    var delegate: GADMediationInterstitialAdEventDelegate?
    var interstitialView: TADInterstitial?
    var completionHandler: GADMediationInterstitialLoadCompletionHandler?

    override required init() {
        super.init()
    }
    
    deinit {
        interstitialView?.delegate = nil
        interstitialView = nil
        print("CustomInterstitial - deinit!!")
    }

    func loadInterstitial(for adConfiguration: GADMediationInterstitialAdConfiguration,
        completionHandler: @escaping GADMediationInterstitialLoadCompletionHandler) {
        // 將 adConfiguration 物件裡的資訊轉入 TADRequest 物件
        let tadRequest = TADRequest()
        tadRequest.isTesting = adConfiguration.isTestRequest
        tadRequest.setLocationWithLatitude(adConfiguration.userLatitude, longitude: adConfiguration.userLongitude, accuracy: adConfiguration.userLocationAccuracyInMeters)
        
        self.completionHandler = completionHandler
        self.interstitialView = TADInterstitial()
        self.interstitialView?.delegate = self
        self.interstitialView?.adUnitID = adConfiguration.credentials.settings["parameter"] as? String
        self.interstitialView?.turnOnMicrophone = false
        self.interstitialView?.load(tadRequest)
    }
    
    func present(from viewController: UIViewController) {
        interstitialView?.present(fromRootViewController: viewController)
    }
}

// MARK: TADInterstitialDelegate

extension TADCustomEventInterstitial: TADInterstitialDelegate {
    func interstitialDidReceiveAd(_ ad: TADInterstitial!) {
        print("CustomInterstitial - interstitialDidReceiveAd!!")
        if let handler = completionHandler {
            delegate = handler(self, nil)
        }
    }

    func interstitial(_ ad: TADInterstitial!, didFailToReceiveAdWithError error: TADRequestError!) {
        print("CustomInterstitial - didFailToReceiveAdWithError!! error: \(String(describing: error))")
        if let handler = completionHandler {
            delegate = handler(nil, error)
        }
        // 需手動設為 nil，auto release 在 admob SDK 會造成在未在 main thread 執行
        self.interstitialView?.delegate = nil
        self.interstitialView = nil
    }

    func interstitialWillPresentScreen(_ ad: TADInterstitial!) {
        print("CustomInterstitial - interstitialWillPresentScreen!!")
        delegate?.reportImpression()
        delegate?.willPresentFullScreenView()
    }

    func interstitialDidDismissScreen(_ ad: TADInterstitial!) {
        print("CustomInterstitial - interstitialWillDismissScreen!!")
        delegate?.didDismissFullScreenView()
    }

    func interstitialWillLeaveApplication(_ ad: TADInterstitial!) {
        print("CustomInterstitial - interstitialWillLeaveApplication!!")
        delegate?.reportClick()
    }
}

