//
//  BannerViewController.swift
//  AdmobSDKTest
//
//  Created by Mira on 2021/6/16.
//

import UIKit
import GoogleMobileAds

class BannerViewController: UIViewController, GADBannerViewDelegate {
    
    @IBOutlet weak var logTextView: UITextView!
    var bannerView: GADBannerView!
    var adType: AdType
    var adUnitId: String

    init(adType: AdType, adUnitId: String) {
        self.adType = adType
        self.adUnitId = adUnitId
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.setupBannerView()
    }
    
    private func setupBannerView() {
        var adSize = GADAdSizeBanner
        switch adType {
        case .banner320X50:
            self.bannerView = GADBannerView(adSize: adSize)
            self.bannerView.adUnitID = adUnitId
        case .banner1200x627:
            adSize = GADAdSizeFromCGSize(CGSize(width: UIScreen.main.bounds.width, height: UIScreen.main.bounds.width * 627/1200))
            self.bannerView = GADBannerView(adSize: adSize)
            self.bannerView.adUnitID = adUnitId
        case .banner300x250:
            adSize = GADAdSizeMediumRectangle
            self.bannerView = GADBannerView(adSize: adSize)
            self.bannerView.adUnitID = adUnitId
        default:
            showLog(with: "TAmedia banner failed to load due to invalid ad size.")
        }
        self.addBannerViewToView(self.bannerView)
        self.bannerView.rootViewController = self
        self.bannerView.delegate = self
    }
    
    private func addBannerViewToView(_ bannerView: GADBannerView) {
        self.bannerView.translatesAutoresizingMaskIntoConstraints = false
        view.addSubview(bannerView)
        view.addConstraints(
            [NSLayoutConstraint(item: bannerView,
                                attribute: .bottom,
                                relatedBy: .equal,
                                toItem: bottomLayoutGuide,
                                attribute: .top,
                                multiplier: 1,
                                constant: 0),
             NSLayoutConstraint(item: bannerView,
                                attribute: .centerX,
                                relatedBy: .equal,
                                toItem: view,
                                attribute: .centerX,
                                multiplier: 1,
                                constant: 0)
            ])
     }
    
    @IBAction func showAd(_ sender: UIButton) {
        self.bannerView.load(GADRequest())
        self.showLog(with: "ad will fetch")
    }
    
    private func showLog(with text:String) {
        logTextView.text.append("\(text) \r\n")
    }
}

//MARK: GADBannerViewDelegate
extension BannerViewController {
    func bannerViewDidReceiveAd(_ bannerView: GADBannerView) {
        print("bannerViewDidReceiveAd")
        if let adResponseInfo = bannerView.responseInfo {
            showLog(with: "response info:\(adResponseInfo)")
        }
        showLog(with: "bannerViewDidReceiveAd")
    }

    func bannerView(_ bannerView: GADBannerView, didFailToReceiveAdWithError error: Error) {
        print("bannerView:didFailToReceiveAdWithError: \(error.localizedDescription)")
        showLog(with: "bannerView:didFailToReceiveAdWithError: \(error.localizedDescription)")
    }

    func bannerViewDidRecordImpression(_ bannerView: GADBannerView) {
        print("bannerViewDidRecordImpression")
        showLog(with: "bannerViewDidRecordImpression")
    }

    func bannerViewWillPresentScreen(_ bannerView: GADBannerView) {
        print("bannerViewWillPresentScreen")
        showLog(with: "bannerViewWillPresentScreen")
    }

    func bannerViewWillDismissScreen(_ bannerView: GADBannerView) {
        print("bannerViewWillDIsmissScreen")
        showLog(with: "bannerViewWillDIsmissScreen")
    }

    func bannerViewDidDismissScreen(_ bannerView: GADBannerView) {
        print("bannerViewDidDismissScreen")
        showLog(with: "bannerViewDidDismissScreen")
    }
}
