//
//  GAMBannerViewController.swift
//  AdmobSDKTest
//
//  Created by Charles Wang on 2023/7/11.
//

import UIKit
import GoogleMobileAds

class GAMBannerViewController: UIViewController, GADBannerViewDelegate {

    @IBOutlet weak var logTextView: UITextView!
    var bannerView: GAMBannerView!
    var adType: AdType
    var adUnitId: String
    
    init(adType: AdType, adUnitId: String) {
        self.adType = adType
        self.adUnitId = adUnitId
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setupBannerView()
    }
    private func setupBannerView() {
        var adSize = GADAdSizeBanner
        switch adType {
        case .banner320X50:
            self.bannerView = GAMBannerView(adSize: adSize)
            self.bannerView.adUnitID = AdUnitID.GAM.banner320x50
        case .banner300x250:
            adSize = GADAdSizeMediumRectangle
            self.bannerView = GAMBannerView(adSize: adSize)
            self.bannerView.adUnitID = AdUnitID.GAM.banner300x250
        default:
            showLog(with: "TAmedia banner failed to load due to invalid ad size.")
        }
        self.addBannerViewToView(self.bannerView)
        self.bannerView.rootViewController = self
        self.bannerView.delegate = self
    }

    func addBannerViewToView(_ bannerView: GAMBannerView) {
      bannerView.translatesAutoresizingMaskIntoConstraints = false
      view.addSubview(bannerView)
      view.addConstraints(
        [NSLayoutConstraint(item: bannerView,
                            attribute: .bottom,
                            relatedBy: .equal,
                            toItem: bottomLayoutGuide,
                            attribute: .bottom,
                            multiplier: 1,
                            constant: 0),
         NSLayoutConstraint(item: bannerView,
                            attribute: .centerX,
                            relatedBy: .equal,
                            toItem: view,
                            attribute: .centerX,
                            multiplier: 1,
                            constant: 0)
        ])
     }
    
    
    @IBAction func showAd(_ sender: Any) {
        print("class name:\(NSStringFromClass(TADCustomEvent.self))")
        
        bannerView.load(GADRequest())
        self.showLog(with: "ad will fetch")
    }
    
    private func showLog(with text:String) {
        logTextView.text.append("\(text) \r\n")
    }
    
    func bannerViewDidReceiveAd(_ bannerView: GADBannerView) {
        self.showLog(with: "ad received")
    }
    
    func bannerView(_ bannerView: GADBannerView, didFailToReceiveAdWithError error: Error) {
        self.showLog(with: "didFailToReceiveAdWithError:\(error.localizedDescription)")
        print("error:\(error)")
    }
}

