//
//  GAMInterstitialViewController.swift
//  AdmobSDKTest
//
//  Created by Charles Wang on 2023/7/12.
//

import UIKit
import GoogleMobileAds

class GAMInterstitialViewController: UIViewController {
    @IBOutlet weak var logTextView: UITextView!
    private var interstitial: GAMInterstitialAd?
    var adUnitId: String
    
    init(adUnitId: String) {
        self.adUnitId = adUnitId
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }

    private func loadInterstitial() {
        self.showLog(with: "ad will fetch")
        let request = GAMRequest()
//        /21685226790/GAM_Mediation_iOS_Interstitial_Test
//        /6499/example/interstitial
        GAMInterstitialAd.load(withAdManagerAdUnitID: AdUnitID.GAM.interstitial,
                                                 request: request,
                                       completionHandler: { [self] ad, error in
                                if let error = error {
                                  print("Failed to load interstitial ad with error: \(error.localizedDescription)")
                                    showLog(with: "Failed to load interstitial ad with error: \(error.localizedDescription)")
                                  return
                                }
                                interstitial = ad
                                interstitial?.fullScreenContentDelegate = self
                                interstitial?.present(fromRootViewController: self)
                              }
            )

    }
    
    @IBAction func showAd(_ sender: Any) {
        if interstitial != nil {
            interstitial?.fullScreenContentDelegate = nil
            interstitial = nil
        }
        loadInterstitial()
    }
    
    private func showLog(with text:String) {
        logTextView.text.append("\(text) \r\n")
    }
   

}

//MARK: GADFullScreenContentDelegate
extension GAMInterstitialViewController: GADFullScreenContentDelegate {
    
    
    func adDidRecordClick(_ ad: GADFullScreenPresentingAd) {
        print("Ad did record click")
        self.showLog(with: "Ad did fail to present full screen content.")
    }
    
    
    func adDidRecordImpression(_ ad: GADFullScreenPresentingAd) {
        print("Ad did record Inpresssion")
        self.showLog(with: "Ad did record Inpresssion")
    }
    
    
    
    /// Tells the delegate that the ad failed to present full screen content.
    func ad(_ ad: GADFullScreenPresentingAd, didFailToPresentFullScreenContentWithError error: Error) {
        print("Ad did fail to present full screen content.")
        self.showLog(with: "Ad did fail to present full screen content.")
    }

    func adWillPresentFullScreenContent(_ ad: GADFullScreenPresentingAd) {
        print("Ad did will show full screen content.")
        self.showLog(with: "Ad did fail to present full screen content.")
    }

    /// Tells the delegate that the ad dismissed full screen content.
    func adDidDismissFullScreenContent(_ ad: GADFullScreenPresentingAd) {
        print("Ad did dismiss full screen content.")
        self.showLog(with: "Ad did dismiss full screen content.")
    }
}
