//
//  InterstitialViewController.swift
//  AdmobSDKTest
//
//  Created by Mira on 2021/6/17.
//

import UIKit
import GoogleMobileAds

class InterstitialViewController: UIViewController, GADFullScreenContentDelegate {
    @IBOutlet weak var logTextView: UITextView!
    var interstitialView: GADInterstitialAd?
    var adUnitId: String
    
    init(adUnitId: String) {
        self.adUnitId = adUnitId
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.title = "Interstitial"
    }

    private func loadInterstitial() {
        self.showLog(with: "ad will fetch")
        let request = GADRequest()
        GADInterstitialAd.load(withAdUnitID:adUnitId,
                                    request: request,
                          completionHandler: { [self] ad, error in
                            if let error = error {
                                print("Failed to load interstitial ad with error: \(error.localizedDescription)")
                                self.showLog(with: "Failed to load interstitial ad with error: \(error.localizedDescription)")
                                return
                            }
                            self.showLog(with: "ad did receive")
                            interstitialView = ad
                            interstitialView?.fullScreenContentDelegate = self
                            interstitialView?.present(fromRootViewController: self)
                          }
        )

    }
    
    private func showLog(with text:String) {
        logTextView.text.append("\(text) \r\n")
    }
    
    @IBAction func showAd(_ sender: UIButton) {
        if interstitialView != nil {
            interstitialView?.fullScreenContentDelegate = nil
            interstitialView = nil
        }
        self.loadInterstitial()
    }
}

//MARK: GADFullScreenContentDelegate
extension InterstitialViewController {
    /// Tells the delegate that the ad failed to present full screen content.
    func ad(_ ad: GADFullScreenPresentingAd, didFailToPresentFullScreenContentWithError error: Error) {
        print("Ad did fail to present full screen content.")
        self.showLog(with: "Ad did fail to present full screen content.")
    }

    /// Tells the delegate that the ad presented full screen content.
    func adDidPresentFullScreenContent(_ ad: GADFullScreenPresentingAd) {
        print("Ad did present full screen content.")
        self.showLog(with: "Ad did present full screen content.")
    }

    /// Tells the delegate that the ad dismissed full screen content.
    func adDidDismissFullScreenContent(_ ad: GADFullScreenPresentingAd) {
        print("Ad did dismiss full screen content.")
        self.showLog(with: "Ad did dismiss full screen content.")
    }
}
