//
//  NativeAdViewController.swift
//  AdmobSDKTest
//
//  Created by Charles Wang on 2022/1/17.
//

import GoogleMobileAds
import UIKit

class NativeAdViewController: UIViewController {
    @IBOutlet var nativeAdPlaceholder: UIView!
    @IBOutlet var logTextView: UITextView!
    var scrollView: UIScrollView?
    
    var nativeAdView: GADNativeAdView!
    var adLoader: GADAdLoader!
    var adUnitId: String
    
    init(adUnitId: String) {
        self.adUnitId = adUnitId
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }

    private func setAdView(_ view: GADNativeAdView) {
        // Remove the previous ad view.
        if nativeAdView != nil {
            nativeAdView.removeFromSuperview()
        }
        nativeAdView = view
        nativeAdPlaceholder.addSubview(nativeAdView)
        nativeAdView.translatesAutoresizingMaskIntoConstraints = false
        nativeAdView.topAnchor.constraint(equalTo: nativeAdPlaceholder.topAnchor).isActive = true
        nativeAdView.leadingAnchor.constraint(equalTo: nativeAdPlaceholder.leadingAnchor).isActive = true
        nativeAdView.trailingAnchor.constraint(equalTo: nativeAdPlaceholder.trailingAnchor).isActive = true
        nativeAdView.bottomAnchor.constraint(equalTo: nativeAdPlaceholder.bottomAnchor).isActive = true
    }
    
    private func setAdToScrollView(_ view: GADNativeAdView) {
        if nativeAdView != nil {
            nativeAdView.removeFromSuperview()
        }
        scrollView?.removeFromSuperview()
        let scrollView = UIScrollView(frame: .zero)
        scrollView.translatesAutoresizingMaskIntoConstraints = false
        let stackView = UIStackView(frame: .zero)
        stackView.translatesAutoresizingMaskIntoConstraints = false
        stackView.axis = .vertical
        stackView.alignment = .center
        scrollView.addSubview(stackView)
        self.view.addSubview(scrollView)
        
        scrollView.topAnchor.constraint(equalTo: nativeAdPlaceholder.topAnchor).isActive = true
        scrollView.leadingAnchor.constraint(equalTo: self.view.leadingAnchor).isActive = true
        scrollView.trailingAnchor.constraint(equalTo: self.view.trailingAnchor).isActive = true
        scrollView.bottomAnchor.constraint(equalTo: self.view.bottomAnchor).isActive = true
        
        stackView.topAnchor.constraint(equalTo: scrollView.topAnchor).isActive = true
        stackView.leadingAnchor.constraint(equalTo: scrollView.leadingAnchor).isActive = true
        stackView.trailingAnchor.constraint(equalTo: scrollView.trailingAnchor).isActive = true
        stackView.bottomAnchor.constraint(equalTo: scrollView.bottomAnchor).isActive = true
        self.scrollView = scrollView
        
        let topSpaceView = UIView(frame: .zero)
        topSpaceView.widthAnchor.constraint(equalToConstant: UIScreen.main.bounds.width).isActive = true
        topSpaceView.heightAnchor.constraint(equalToConstant: 1000).isActive = true
                
        let label = UILabel(frame: .zero)
        stackView.translatesAutoresizingMaskIntoConstraints = false
        label.widthAnchor.constraint(equalToConstant: UIScreen.main.bounds.width).isActive = true
        label.heightAnchor.constraint(equalToConstant: 50).isActive = true
        label.text = "Scroll up!!"
        label.textAlignment = .center
        stackView.addArrangedSubview(label)
        
        stackView.addArrangedSubview(topSpaceView)
        nativeAdView = view
        view.translatesAutoresizingMaskIntoConstraints = false
        view.widthAnchor.constraint(equalToConstant: 350).isActive = true
        view.heightAnchor.constraint(equalToConstant: 350).isActive = true
        stackView.addArrangedSubview(view)
    }
    
    private func refreshAd() {
        adLoader = GADAdLoader(
            adUnitID: adUnitId, rootViewController: self,
            adTypes: [.native], options: nil)
        adLoader.delegate = self
        adLoader.load(GADRequest())
    }
    
    private func showLog(with text: String) {
        logTextView.text.append("\(text) \r\n")
    }
    
    @IBAction func showAd(_ sender: Any) {
        refreshAd()
    }
}

extension NativeAdViewController: GADAdLoaderDelegate, GADNativeAdLoaderDelegate {
    func adLoader(_ adLoader: GADAdLoader, didReceive nativeAd: GADNativeAd) {
        showLog(with: "adLoader did success")
        guard
          let nibObjects = Bundle.main.loadNibNamed("NativeAdView", owner: nil, options: nil),
          let adView = nibObjects.first as? GADNativeAdView
        else {
            assert(false, "Could not load nib file for adView")
            return
        }
        setAdView(adView)
        setAdToScrollView(adView)
        nativeAd.delegate = self
        
        (nativeAdView.headlineView as? UILabel)?.text = nativeAd.headline
        nativeAdView.mediaView?.mediaContent = nativeAd.mediaContent
        
        let mediaContent = nativeAd.mediaContent
        if mediaContent.hasVideoContent {
            mediaContent.videoController.delegate = self
        }
        
        (nativeAdView.bodyView as? UILabel)?.text = nativeAd.body
        nativeAdView.bodyView?.isHidden = nativeAd.body == nil

        (nativeAdView.callToActionView as? UIButton)?.setTitle(nativeAd.callToAction, for: .normal)
        nativeAdView.callToActionView?.isHidden = nativeAd.callToAction == nil

        (nativeAdView.iconView as? UIImageView)?.image = nativeAd.icon?.image
        nativeAdView.iconView?.isHidden = nativeAd.icon == nil
        
        nativeAdView.starRatingView?.isHidden = nativeAd.starRating == nil

        (nativeAdView.storeView as? UILabel)?.text = nativeAd.store
        nativeAdView.storeView?.isHidden = nativeAd.store == nil

        (nativeAdView.priceView as? UILabel)?.text = nativeAd.price
        nativeAdView.priceView?.isHidden = nativeAd.price == nil

        (nativeAdView.advertiserView as? UILabel)?.text = nativeAd.advertiser
        nativeAdView.advertiserView?.isHidden = nativeAd.advertiser == nil
        nativeAdView.callToActionView?.isUserInteractionEnabled = false
        nativeAdView.nativeAd = nativeAd
    }
    
    func adLoader(_ adLoader: GADAdLoader, didFailToReceiveAdWithError error: Error) {
        showLog(with: "adLoader didFail")
    }
}

extension NativeAdViewController: GADNativeAdDelegate {
    func nativeAdDidRecordClick(_ nativeAd: GADNativeAd) {
        print("gad nativeAdDidRecordClick")
    }
    
    func nativeAdDidRecordImpression(_ nativeAd: GADNativeAd) {
        print("gad nativeAdDidRecordImpression")
    }
}

extension NativeAdViewController: GADVideoControllerDelegate {
    func videoControllerDidPlayVideo(_ videoController: GADVideoController) {
        print("gad videoControllerDidPlayVideo")
    }
    
    func videoControllerDidPauseVideo(_ videoController: GADVideoController) {
        print("gad videoControllerDidPauseVideo")
    }
    
    func videoControllerDidEndVideoPlayback(_ videoController: GADVideoController) {
        print("gad videoControllerDidEndVideoPlayback")
    }
    
    func videoControllerDidMuteVideo(_ videoController: GADVideoController) {
        print("gad videoControllerDidMuteVideo")
    }
    
    func videoControllerDidUnmuteVideo(_ videoController: GADVideoController) {
        print("gad videoControllerDidUnmuteVideo")
    }
}
